@@ -8,19 +8,27 @@
 #' `start_capturing` and `stop_capturing` allow you to turn on/off request
 #' recording for more convenient use in an interactive session.
 #'
-#' Mocks stored by this context are written out as plain-text files, either with
-#' extension `.json` if the request returned JSON content or with extension `.R`
-#' otherwise. The `.R` files contain syntax that when executed recreates the
-#' `httr` "response" object. By storing fixtures as plain-text files, you can
+#' Recorded responses are written out as plain-text files. By storing fixtures
+#' as plain-text files, you can
 #' more easily confirm that your mocks look correct, and you can more easily
 #' maintain them without having to re-record them. If the API changes subtly,
 #' such as when adding an additional attribute to an object, you can just touch
 #' up the mocks.
 #'
+#' If the response has status `200 OK` and the `Content-Type`
+#' maps to a supported file extension---currently `.json`,
+#' `.html`, `.xml`, `.txt`, `.csv`, and `.tsv`---just the response body will be
+#' written out, using the appropriate extension. `204 No Content` status
+#' responses will be stored as an empty file with extension `.204`. Otherwise,
+#' the response will be written as a `.R` file containing syntax that, when
+#' executed, recreates the
+#' `httr` "response" object.
+#'
 #' @param expr Code to run inside the context
 #' @param path Where to save the mock files. Default is the first directory in
 #' [.mockPaths()], which if not otherwise specified is the current working
-#' directory.
+#' directory. It is generally better to call `.mockPaths()` directly if you
+#' want to write to a different path, rather than using the `path` argument.
 #' @param simplify logical: if `TRUE` (default), JSON responses with status 200
 #' will be written as just the text of the response body. In all other cases,
 #' and when `simplify` is `FALSE`, the "response" object will be written out to
@@ -28,7 +36,9 @@
 #' @param verbose logical: if `TRUE`, a `message` is printed for every file
 #' that is written when capturing requests containing the absolute path of the
 #' file. Useful for debugging if you're capturing but don't see the fixture
-#' files being written in the expected location. Default is `FALSE`.
+#' files being written in the expected location. Default is `FALSE`. This
+#' parameter is deprecated; it is instead recommended that you set
+#' `options(httptest.verbose=TRUE)` to enable.
 #' @param redact function to run to purge sensitive strings from the recorded
 #' response objects. This argument is deprecated: use [set_redactor()] or a
 #' package redactor instead. See `vignette("redacting")` for more details.
@@ -54,6 +64,7 @@
 #' }
 #' @importFrom httr content
 #' @export
+#' @seealso [build_mock_url()] for how requests are translated to file paths
 capture_requests <- function (expr, path, ...) {
     start_capturing(...)
     on.exit(stop_capturing())
@@ -67,13 +78,19 @@ capture_requests <- function (expr, path, ...) {
 
 #' @rdname capture_requests
 #' @export
-start_capturing <- function (path, simplify=TRUE, verbose=FALSE, redact) {
+start_capturing <- function (path, simplify=TRUE, verbose, redact) {
     if (!missing(path)) {
         ## Note that this changes state and doesn't reset it
         .mockPaths(path)
     } else {
         path <- NULL
     }
+    if (!missing(verbose)) {
+        ## Note that this changes state and doesn't reset it
+        warning("The 'verbose' argument to capture_requests() is deprecated; ",
+            "Set options(httptest.verbose=TRUE) instead.", call.=FALSE)
+        options(httptest.verbose=verbose)
+    }
 
     if (!missing(redact)) {
         warning("The 'redact' argument to start_capturing() is deprecated. ",
@@ -86,13 +103,10 @@ start_capturing <- function (path, simplify=TRUE, verbose=FALSE, redact) {
         ## Get the value returned from the function, and sanitize it
         redactor <- get_current_redactor()
         .resp <- redactor(returnValue())
-        f <- save_response(.resp, simplify=simplify)
-        if (isTRUE(getOption("httptest.verbose", verbose))) {
-            message("Writing ", normalizePath(f))
-        }
-    }, list(simplify=simplify, verbose=verbose))
-    for (verb in c("PUT", "POST", "PATCH", "DELETE", "VERB", "GET")) {
-        trace_httr(verb, exit=req_tracer, print=FALSE)
+        save_response(.resp, simplify=simplify)
+    }, list(simplify=simplify))
+    for (verb in c("PUT", "POST", "PATCH", "DELETE", "VERB", "GET", "RETRY")) {
+        trace_httr(verb, exit=req_tracer)
     }
     invisible(path)
 }
@@ -101,34 +115,43 @@ start_capturing <- function (path, simplify=TRUE, verbose=FALSE, redact) {
 #'
 #' @param response An 'httr' `response` object
 #' @param simplify logical: if `TRUE` (default), JSON responses with status 200
+#' and a supported `Content-Type`
 #' will be written as just the text of the response body. In all other cases,
 #' and when `simplify` is `FALSE`, the "response" object will be written out to
 #' a .R file using [base::dput()].
 #' @return The character file name that was written out
 #' @export
 #' @keywords internal
+#' @importFrom jsonlite prettify
 save_response <- function (response, simplify=TRUE) {
     ## Construct the mock file path
-    mapped_file <- buildMockURL(response$request)
-    filename <- file.path(.mockPaths()[1], mapped_file)
-    dir.create(dirname(filename), showWarnings=FALSE, recursive=TRUE)
-
-    ## Omit curl handle C pointer, which doesn't serialize meaningfully
-    response$handle <- NULL
+    mock_file <- buildMockURL(response$request)
+    ## Track separately the actual full path we're going to write to
+    dst_file <- file.path(.mockPaths()[1], mock_file)
+    dir.create(dirname(dst_file), showWarnings=FALSE, recursive=TRUE)
 
     ## Get the Content-Type
-    ct <- unlist(response$headers[tolower(names(response$headers)) == "content-type"])
-    is_json <- any(grepl("application/json", ct))
-    if (simplify && response$status_code == 200 && is_json) {
+    ct <- get_content_type(response)
+    status <- response$status_code
+    if (simplify && status == 200 && ct %in% names(CONTENT_TYPE_TO_EXT)) {
         ## Squelch the "No encoding supplied: defaulting to UTF-8."
-        ## TODO: support other text content-types than JSON
-        cat(suppressMessages(content(response, "text")), file=filename)
+        cont <- suppressMessages(content(response, "text"))
+        if (ct == "application/json") {
+            ## Prettify
+            cont <- prettify(cont)
+        }
+        dst_file <- paste(dst_file, CONTENT_TYPE_TO_EXT[[ct]], sep=".")
+        cat(cont, file=dst_file)
+    } else if (simplify && status == 204) {
+        ## "touch" a file with extension .204
+        dst_file <- paste0(dst_file, ".204")
+        cat("", file=dst_file)
     } else {
         ## Dump an object that can be sourced
 
         ## Change the file extension to .R
-        filename <- sub("json$", "R", filename)
-        mapped_file <- sub("json$", "R", mapped_file)
+        dst_file <- paste0(dst_file, ".R")
+        mock_file <- paste0(mock_file, ".R")
 
         ## If content is text, rawToChar it and dput it as charToRaw(that)
         ## so that it loads correctly but is also readable
@@ -136,11 +159,13 @@ save_response <- function (response, simplify=TRUE) {
             "application/x-www-form-urlencoded", "application/xml",
             "text/csv", "text/html", "text/plain",
             "text/tab-separated-values", "text/xml")
-        is_text <- length(ct) && any(unlist(strsplit(ct, "; ")) %in% text_types)
-        ## strsplit on ; because "charset" may be appended
-        if (is_text) {
+        if (ct %in% text_types) {
             ## Squelch the "No encoding supplied: defaulting to UTF-8."
             cont <- suppressMessages(content(response, "text"))
+            # if (ct == "application/json") {
+            #     ## TODO: "parse error: premature EOF"
+            #     cont <- jsonlite::prettify(cont)
+            # }
             response$content <- substitute(charToRaw(cont))
         } else if (inherits(response$request$output, "write_disk")) {
             ## Copy real file and substitute the response$content "path".
@@ -148,21 +173,30 @@ save_response <- function (response, simplify=TRUE) {
             ## make the mock file readable call `content()`, which reads
             ## in the file that has been written to disk, so it effectively
             ## negates the "download" behavior for the recorded response.
-            downloaded_file <- paste0(filename, "-FILE")
+            downloaded_file <- paste0(dst_file, "-FILE")
             file.copy(response$content, downloaded_file)
-            mapped_file <- paste0(mapped_file, "-FILE")
-            response$content <- substitute(structure(find_mock_file(mapped_file),
+            mock_file <- paste0(mock_file, "-FILE")
+            response$content <- substitute(structure(find_mock_file(mock_file),
                 class="path"))
         }
-        dput(response, file=filename)
+
+        ## Omit curl handle C pointer, which doesn't serialize meaningfully
+        response$handle <- NULL
+        ## Drop request since httr:::request_perform will fill it in when loading
+        response$request <- NULL
+
+        dput(response, file=dst_file)
     }
-    return(filename)
+    if (isTRUE(getOption("httptest.verbose", FALSE))) {
+        message("Writing ", normalizePath(dst_file))
+    }
+    return(dst_file)
 }
 
 #' @rdname capture_requests
 #' @export
 stop_capturing <- function () {
-    for (verb in c("GET", "PUT", "POST", "PATCH", "DELETE", "VERB")) {
+    for (verb in c("GET", "PUT", "POST", "PATCH", "DELETE", "VERB", "RETRY")) {
         safe_untrace(verb, add_headers)
     }
 }
