@@ -14,11 +14,6 @@ chain_redactors <- function (funs) {
 #' @rdname redact
 #' @export
 redact_cookies <- function (response) {
-    ## Delete from request
-    if ("cookie" %in% names(response$request$options)) {
-        response$request$options$cookie <- "REDACTED"
-    }
-    ## Delete from response
     response <- redact_headers(response, "Set-Cookie")
     if (!is.null(response$cookies) && nrow(response$cookies)) {
         ## is.null check is for reading mocks.
@@ -31,36 +26,27 @@ redact_cookies <- function (response) {
 #' @rdname redact
 #' @export
 redact_headers <- function (response, headers=c()) {
-    response$headers <- redact_from_header_list(response$headers, headers)
-    response$all_headers <- lapply(response$all_headers, function (h) {
-        h$headers <- redact_from_header_list(h$headers, headers)
-        return(h)
-    })
-    response$request$headers <- redact_from_header_list(response$request$headers,
-        headers)
-    return(response)
+    header_apply(response, headers, function (x) "REDACTED")
 }
 
-redact_from_header_list <- function (headers, to_redact=c()) {
-    bad <- tolower(names(headers)) %in% tolower(to_redact)
-    headers[bad] <- "REDACTED"
-    return(headers)
-}
-
-#' @rdname redact
-#' @export
-redact_http_auth <- function (response) {
-    if ("userpwd" %in% names(response$request$options)) {
-        response$request$options$userpwd <- "REDACTED"
+header_apply <- function (response, headers, FUN, ...) {
+    ## Apply some function over a set of named headers, anywhere they may
+    ## appear in a response or request object
+    response$headers <- happly(response$headers, headers, FUN, ...)
+    if (!is.null(response$all_headers)) {
+        response$all_headers <- lapply(response$all_headers, function (h) {
+            h$headers <- happly(h$headers, headers, FUN, ...)
+            return(h)
+        })
     }
     return(response)
 }
 
-#' @rdname redact
-#' @export
-redact_oauth <- function (response) {
-    response$request$auth_token <- NULL
-    return(redact_headers(response, "Authorization"))
+happly <- function (header_list, headers, FUN, ...) {
+    ## Called from header_apply, actually does the applying on a header list
+    todo <- tolower(names(header_list)) %in% tolower(headers)
+    header_list[todo] <- lapply(header_list[todo], FUN, ...)
+    return(header_list)
 }
 
 #' @rdname redact
@@ -102,7 +88,9 @@ within_body_text <- function (response, FUN) {
 gsub_response <- function (response, pattern, replacement, ...) {
     replacer <- function (x) gsub(pattern, replacement, x, ...)
     response$url <- replacer(response$url)
+    response <- header_apply(response, "location", replacer)
     response <- within_body_text(response, replacer)
+    ## Modify the request too because this affects where we write the mock file to
     response$request <- gsub_request(response$request, pattern, replacement, ...)
     return(response)
 }
@@ -169,9 +157,7 @@ as.redactor <- function (fmla) {
 #'
 #' `redact_cookies()` removes cookies from 'httr' `response` objects.
 #' `redact_headers()` lets you target selected request and response headers for
-#' redaction. `redact_http_auth()` removes `username:password`-based HTTP auth
-#' credentials. `redact_oauth()` removes the OAuth 'Token' object that 'httr'
-#' sticks in the request object. `redact_auth()` is a convenience wrapper around
+#' redaction. `redact_auth()` is a convenience wrapper around
 #' them for a useful default redactor in `capture_requests()`.
 #'
 #' `within_body_text()` lets you manipulate the text of the response body
@@ -187,12 +173,7 @@ as.redactor <- function (fmla) {
 #' @return All redacting functions return a well-formed 'httr' `response`
 #' object.
 #' @name redact
-#' @aliases redact_auth redact_cookies redact_headers redact_http_auth redact_oauth within_body_text
+#' @aliases redact_auth redact_cookies redact_headers within_body_text
 #' @seealso `vignette("redacting", package="httptest")` for a detailed discussion of what these functions do and how to customize them. [gsub_response()] is another redactor.
 #' @export
-redact_auth <- chain_redactors(list(
-    redact_cookies,
-    ~ redact_headers(., c("Authorization", "Proxy-Authorization")),
-    redact_http_auth,
-    redact_oauth
-))
+redact_auth <- redact_cookies
