@@ -45,12 +45,16 @@ stop_request <- function (req, handle, refresh) {
     out <- paste(req$method, req$url)
     body <- request_body(req)
     if (!is.null(body)) {
+        ## Max print option for debugging large payloads
+        body <- substr(body, 1, getOption("httptest.max.print", nchar(body)))
         out <- paste(out, body)
     }
+
     if (!is.null(req$mockfile)) {
         ## Poked in here by mock_request for ease of debugging
         ## Append it to the end.
-        out <- paste0(out, " (", req$mockfile, ")")
+        ## TODO: remove .json if/when possible; there for backwards compat
+        out <- paste0(out, " (", req$mockfile, ".json)")
     }
     stop(out, call.=FALSE)
 }
