@@ -1,6 +1,9 @@
 Sys.setlocale("LC_COLLATE", "C") ## What CRAN does
 set.seed(999)
-options(warn=1)
+options(
+    warn=1,
+    httptest.debug=FALSE
+)
 
 library(httr)
 
@@ -29,6 +32,9 @@ if ("Rcmd" %in% ls(envir=asNamespace("tools"))) {
     }
 }
 
-install_testpkg <- function (pkg) {
-    Rcmd(c("INSTALL", pkg))
+install_testpkg <- function (pkg, lib=tempfile()) {
+    dir.create(lib)
+    Rcmd(c("INSTALL", "testpkg", paste0("--library=", shQuote(lib))),
+        stdout=NULL, stderr=NULL)
+    return(lib)
 }
