@@ -5,11 +5,14 @@ public({
         expect_identical(.mockPaths(), ".")
         .mockPaths("something else")
         expect_identical(.mockPaths(), c("something else", "."))
+        # Unique paths
+        .mockPaths(".")
+        expect_identical(.mockPaths(), c(".", "something else"))
         .mockPaths(NULL)
         expect_identical(.mockPaths(), ".")
     })
 
-    with_mock_API({
+    with_mock_api({
         test_that("GET with no query, default mock path", {
             b <- GET("api/object1/")
             expect_identical(content(b), list(object=TRUE))
